;Same as tutorial 04, but using the zlib.lib to compress the pdf file

#PurePDF_Include=1
XIncludeFile "PurePDF.pb"

Structure s_Header
  Title.s
EndStructure

Structure s_Footer
  Footer.s
EndStructure

Define Header.s_Header
Define Footer.s_Footer

Global col.f
Global y0.f

Header\Title="20000 Leagues Under the Seas"
Footer\Footer="Page "

CompilerSelect #PB_Compiler_OS
  CompilerCase #PB_OS_Linux
    #ZLIB_IMPORT_PATH = #PB_Compiler_Home + "purelibraries/linux/libraries/zlib.a"
  CompilerCase #PB_OS_Windows
    #ZLIB_IMPORT_PATH = "zlib.lib"
  CompilerCase #PB_OS_MacOS
    #ZLIB_IMPORT_PATH = "/usr/lib/libz.dylib"
CompilerEndSelect

ImportC #ZLIB_IMPORT_PATH
  CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
    _compress.l(*dest, *destlen, *source, sourcelen) As "_compress"
  CompilerElse
    _compress.l(*dest, *destlen, *source, sourcelen) As "compress"
  CompilerEndIf
EndImport

Procedure.l PDFCompress(*aData.MEM_DataStructure)
  Protected zerr,CompMem,SourceLen,SourceMem,CompLen,lbError
  
  lbError = #False
  SourceLen = *aData\lCurSize
  SourceMem = *aData\pData 
  CompLen   = Round(1.001 * SourceLen,#PB_Round_Up) + 12
  CompMem   = AllocateMemory(CompLen)
  If CompMem
    zerr=_compress(CompMem,@CompLen,SourceMem,SourceLen)
    If zerr = 0
      *aData\pData = CompMem
      *aData\lMaxSize = CompLen
      *aData\lCurSize = CompLen
      FreeMemory(Sourcemem)
      lbError = #True 
    EndIf
  EndIf
  ProcedureReturn lbError  
EndProcedure

Procedure Header(*Header.s_Header)
  Protected w
  pdf_SetFont("Arial","B",15)
  w = pdf_GetStringWidth(*Header\Title)+6
  pdf_SetX((210 - w)/2)
  pdf_SetDrawColor(0,80,180)
  pdf_SetFillColor(230,230,0)
  pdf_SetTextColor(220,50,50)
  pdf_SetLineWidth(1)
  pdf_Cell(w,9,*Header\Title,1,1,#PDF_ALIGN_CENTER,1)
  pdf_Ln(10)
  y0 = pdf_GetY()
EndProcedure

Procedure Footer(*Footer.s_Footer)
  Protected txt.s
  pdf_SetY(-15)
  pdf_SetFont("Arial","I",8)
  pdf_SetTextColor(128)
  txt.s = *Footer\Footer + Str(pdf_GetPageNo()) + " / {nb}"
  pdf_Cell(0,10,txt,0,0,#PDF_ALIGN_CENTER)
EndProcedure

Procedure SetCol(cl.f)
  Protected x
  ;Set position at a given column
  col = cl;
  x=10+cl*65;
  pdf_SetLeftMargin(x);
  pdf_SetX(x);
EndProcedure

Procedure AcceptPageBreak()
  ;Method accepting Or not automatic page Break
  If(col<2)
    ;Go To Next column
    SetCol(col+1)
    ;Set ordinate To top
    pdf_SetY(y0)
    ;Keep on page
    ProcedureReturn #False
  Else
    ;Go back To first column
    SetCol(0)
    ;Page Break
    ProcedureReturn #True
  EndIf
EndProcedure

Procedure ChapterTitle(num, label.s)
  pdf_SetFont("Arial", "",12)
  pdf_SetFillColor(200,220,255)
  pdf_Cell(0,6,"Chapter " + Str(num) + " : " + label,0,1,#PDF_ALIGN_LEFT,1)
  pdf_Ln(4)
  y0=pdf_GetY()
EndProcedure

Procedure ChapterBody(file.s)
  Protected txt.s,l,*MemoryBuffer
  OpenFile(0,file)
  l = FileSize(file)
  *MemoryBuffer = AllocateMemory(l+2) 
  
  l = ReadData(0,*MemoryBuffer, l)
  CloseFile(0)
  txt = PeekS(*MemoryBuffer ,l,#PB_Ascii) 
  pdf_SetFont("Times","",12)
  pdf_MultiCell(60,5,txt)
  pdf_Ln()
  pdf_SetFont("","I")
  pdf_Cell(0,5,"(end of excerpt)")
  setcol(0)
EndProcedure

Procedure PrintChapter(num,atitle.s,file.s)
  pdf_AddPage()
  ChapterTitle(num,atitle)
  ChapterBody(file)
EndProcedure

pdf_Create()
pdf_AliasNbPages()
pdf_SetProcHeader(@Header(),Header)
pdf_SetProcFooter(@Footer(),Footer)
pdf_SetProcCompression(@PDFCompress())
pdf_SetProcAcceptPageBreak(@AcceptPageBreak())
pdf_SetTitle(Header\Title)
pdf_SetAuthor("Jules Verne")

PrintChapter(1,"A RUNAWAY REEF","20k_c1.txt");
PrintChapter(2,"THE PROS And CONS","20k_c2.txt");
Define File.s="Tutorial05 Compressed.pdf"
pdf_Save(File)
RunProgram(File)

; IDE Options = PureBasic 5.21 LTS (Windows - x64)
; CursorPosition = 38
; FirstLine = 18
; Folding = --
; EnableUnicode